// smart_assert_defs.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_DEFS_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_DEFS_HPP_INCLUDED

#if _MSC_VER >= 1020
#pragma once
#endif

#include <string>
#include <sstream>
#include <boost/current_function.hpp>
#include "smart_assert_alloc.hpp"

namespace boost { 

namespace smart_assert {


// ... internals
typedef char char_type;
typedef std::basic_string< char_type, std::char_traits< char_type>, Private::smart_assert_alloc< char_type> > string_type;
typedef std::basic_ostringstream< char_type, std::char_traits< char_type>, Private::smart_assert_alloc< char_type> > ostringstream_type;
typedef std::basic_istringstream< char_type, std::char_traits< char_type>, Private::smart_assert_alloc< char_type> > istringstream_type;

// ... what the clients of the library might use
typedef std::basic_string< char_type> client_string_type;
// note: no need for client_*stringsteam_type, since
// stringstreams are used ONLY internally
typedef std::ostream client_ostream_type;
typedef std::istream client_istream_type;
typedef std::iostream client_iostream_type;






/* 
    default ASSERT levels
    (you can add your own, just make sure it does not
    mix with this)

    IMPORTANT
    Convention:

    The bigger the level is, the more critical is the error.
    For instance, a warning has a very low level number.
    The debug level is more critical than a warning, etc.

    As I said, you can add your own custom levels, just make sure
    you stick to this convension.

    Do it because the current code (and future code, for this matter) 
    assumes this of you.
*/
enum {

    // default behavior - just loggs this assert
    // (a message is shown to the user to the console)
    lvl_warn = 100,

    // default behavior - asks the user what to do:
    // Ignore/ Retry/ etc.
    lvl_debug = 200,

    // default behavior - throws a smart_assert_error
    lvl_error = 300,

    // default behavior - dumps all assert context to console, 
    // and aborts
    lvl_fatal = 1000
};


/*
    when ignoring an ASSERTion, how are we ignoring it?
*/
typedef enum ignore_type {
    
    // ignores this ASSERTion now
    // (if it triggers again, it won't be ignored by default)
    ignore_now,

    // ignores this ASSERTion forever 
    // (throughout the lifetime of the program)
    ignore_forever,

    // ignores all ASSERTions from now on
    // (note: they are still logged...)
    ignore_all,

    // ignores all ASSERTions related to this assertion
    // throughout the lifetime of the program
    //
    // NOTE - not implemented yet
    ignore_all_related,


    // ignores this ASSERTion forever
    //
    // (if ASSERTions are persisted, this will be ignored throughout
    //  next runs of the same program;
    //
    //  otherwise, behaves like ignore_forever)
    ignore_forever_persistent,


    // ignores all ASSERTions related to this assertion
    // throughout the lifetime of the program
    //
    // (if ASSERTions are persisted, they will be ignored throughout
    //  next runs of the same program;
    //
    //  otherwise, behaves like ignore_all_related)
    //
    // NOTE - not implemented yet
    ignore_all_related_persistent,


    // ignores all ASSERTions from now on
    // (note: they are still logged...)
    //
    // (if ASSERTions are persisted, they will be ignored throughout
    //  next runs of the same program;
    //
    //  otherwise, behaves like ignore_all)
    ignore_all_persistent

    /*
        possible future improvement

        ignore_this_function:
        - ignores all assertions from this function, for the lifetime
          of the program, when this is possible
          (the BOOST_CURRENT_FUNCTION macro)

      */
}; // typedef enum ignore_type 


// note: we're not throwing a std::runtime_exception, since
// you construct a runtime_exception by passing a std::string;
//
// creating a temporary std::string to pass to a runtime_exception,
// in case the system has run out of memory could fail!!!
struct smart_assert_error : public std::exception {
    smart_assert_error( const string_type & val) : m_val( val) {
    };
    smart_assert_error( const char * val) : m_val( val) {
    };

    ~smart_assert_error() throw() {
    }

    const char * what() const throw() {
        return m_val.c_str();
    }
private:
    string_type m_val;
};


/*
    allows initializing the SMART_ASSERT library.

    This way, we force the user to set the logger and handlers/ persistence (if necessary)

 Usage

    In your code, in ONE of the source files (.c*) from your project,
    do something similar to the following:

    #include <boost/smart_assert_ext.hpp>

    // ... whatever

    // note: you can actually copy-and-paste the following 3 lines
    namespace boost { namespace smart_assert {
    smart_assert_initializer initializer( "./assert.txt", "./persist.txt", &default_initialize);
    }}

*/
struct smart_assert_initializer {

    template< class type>
        smart_assert_initializer( const char_type * strLogFileName, const char_type * strPersistFileName, type val) {

        reinitialize( strLogFileName, strPersistFileName, val);
    }

    template< class type>
        smart_assert_initializer( const char_type * strLogFileName, const char_type * strPersistFileName, const char_type * strDefaultConfig, type val) {

        reinitialize( strLogFileName, strPersistFileName, strDefaultConfig, val);
    }


    template< class type>
    void reinitialize( const char_type * strLogFileName, const char_type * strPersistFileName, type val) {

        val( strLogFileName, strPersistFileName, "dbg");
    }

    template< class type>
    void reinitialize( const char_type * strLogFileName, const char_type * strPersistFileName, const char_type * strDefaultConfig, type val) {

        val( strLogFileName, strPersistFileName, strDefaultConfig);
    }

};
extern smart_assert_initializer initializer;



} // namespace smart_assert

} // namespace boost


// note: the user can define SMART_ASSERT_CONTEXT prior to 
// #including any of smart_assert*.hpp files, to use its own!
#ifndef SMART_ASSERT_CONTEXT

    // FIXME (possible improvement)
    // include the thread-id of the current thread, when the assertion fails
    // TOTHINK how will find get it? (in a portable way?)

    #define SMART_ASSERT_CONTEXT context("file",__FILE__).context("line",__LINE__).context("function",BOOST_CURRENT_FUNCTION)

#endif // SMART_ASSERT_CONTEXT


#ifdef WIN32
#define BOOST_SMART_ASSERT_WIN32
#endif

#endif // BOOST_INNER_SMART_ASSERT_DEFS_HPP_INCLUDED
